import logging
from functools import wraps

logging.basicConfig(filename='loglines.txt',level=logging.INFO)

def log(fn):
    @wraps(fn)
    def logged_function(*args, **kwargs):
        name = fn.__name__
        arglist = "%s, " * len(args)
        kwargslist = ["{}=%s, ".format(key) for key in kwargs.keys()]
        kwargsvals = [val for val in kwargs.values()]
        formatted = ("Called {name}("+arglist+''.join(kwargslist)+")").format(name=name)

        format_list = list(args) + list(kwargsvals)
        logging.info(formatted, *format_list)
        fn(*args, **kwargs)
    return logged_function




    ## uses the vision api from google for ocr. it's fairly fast, very accurate.
import base64, cStringIO, requests
url = "https://vision.googleapis.com/v1/images:annotate?key="

def request_body(image):
    return {
        "requests" : [{
            "image": image_obj(image),
            "features": [{"type": "TEXT_DETECTION"}]
        }]
    }

def image_obj(contents):
    bufferr = cStringIO.StringIO()
    contents.save(bufferr, format='JPEG')
    return { "content": base64.b64encode(bufferr.getvalue()) }


def make_recognition(image, key):
    req = requests.post(url+key, json=request_body(image))
    return req.json()["responses"][0]["textAnnotations"][0]["description"]



import pyautogui
import os, sys
from item_object import Item, Items
from functools import wraps
from error_handling import Error

METHODS = { }
currentresults = []
names = os.listdir('./'+sys.argv[0]+'/imgs')
Items = {name: Item(name) for name in names}

class GUI_INTERFACE():
    def __init__(self):
        pass

    def keywrite(self, *args):
        pyautogui.keywrite(list(args))

ui = GUI_INTERFACE()

def new_method(name):
    def new_method_wrapper(fn):
        METHODS[name] = fn
        @wraps(fn)
        def wrapped_method(*args, **kwargs):
            fn(*args, **kwargs)
        return wrapped_method
    return new_method_wrapper


def run(obj=None, cmd=None, cmd_args=None, cmd_kwargs=None):
    METHODS["START"](obj, cmd)
    try:
        currentresults = METHODS[cmd](*cmd_args, **cmd_kwargs)
    except Error as e:
        METHODS["FAILED"](e, obj, cmd)
    else:
        METHODS["COMPLETED"](obj, cmd)




class ItemNotThere(Exception):
    def __init__(self, itemobj):
        context = "Could not find item named {} with image {}".format(
            str(itemobj), itemobj.picturename
        )
        Exception.__init__(self, context)
        self.itemobj = itemobj

class ItemNeverAppeared(Exception):
    def __init__(self, itemobj, funcname):
        context = "Was going to activate function named {} from item {} with \
                   image {}, but it never appeared.".format(
            funcname, str(itemobj), itemobj.picturename
        )
        Exception.__init__(self, context)
        self.itemobj = itemobj

class Error(Exception):
    def __init__(self, errormessage, finishing=[]):
        Exception.__init__(self, errormessage)
        self.log = errormessage
        self.finishing = finishing



import pyautogui as gi
import time, os, ocr
from error_handling import *
from spooky_logs import log

# needed to allow silent failing chains
class Empty:
    def __init__(self):
        pass

    def __call__(self, *args, **kwargs):
        return self

    def __getattr__(self, name):
        return self

class Item(object):
    @log
    def __init__(self, picturename):
        self.picturename = picturename
        self.url = "C:/Casper/casper-worker/worker/images/" + picturename
        self.result = None

    @log
    def __click(self, after=0.2, offx=0, offy=0):
        time.sleep(after)
        pos = gi.locateCenterOnScreen(self.url)
        if pos:
            gi.moveTo(pos[0]+offx, pos[1]+offy)
            gi.click()
            logging.info("Clicked on %s" % self.picturename)
            return self
        else:
            logging.warning("Could not click on %s" % self.picturename)
            raise ItemNotThere(self)

    @log
    def run_when_present(self, func, *args, **kwargs):
        for _ in range(10):
            logging.info("Calling run_when_present, looking for %s",
                          self.picturename)
            if gi.locateCenterOnScreen(self.url) == None:
                logging.info("Did not find image in run_when_present. Will try again.")
                time.sleep(3)
                continue
            logging.info("Found image %s in run_when_present call.",
                         self.picturename)
            logging.info("Executing function %s in run_when_present call for %s",
                         func.__name__, self.picturename)
            self.result = func(*args, **kwargs)
            return self
        logging.warning("Never found %s in run_when_present call.", self.picturename)
        raise ItemNeverAppeared(self, self.run_when_present.__name__)

    @log
    def click(self, offx=0, offy=0):
        return self.run_when_present(self.__click, after=0.2, offx=offx, offy=offy)

    @staticmethod
    @log
    def click_all(*args):
        for i in args:
            Items[i].click()

    @staticmethod
    @log
    def wait_for(*args):
        theres = []
        while True not in theres:
            for i in args:
                theres.append(Items[i].found)
            ui.typewrite(*tkwargs["typewrite_between"])


    @log
    def drag_by(x=0,y=0):
        # to implement #
        pass

    @property
    @log
    def found(self):
        if gi.locateCenterOnScreen(self.url):
            logging.info("Found %s", self.picturename)
            return True
        else:
            logging.info("Did not find %s", self.picturename)
            return False

    @log
    def if_found(self, plus=True, errormsg=''):
        if self.found and plus:
            logging.info("%s was found and plus was true", self.picturename)
            return self
        else:
            logging.info("%s was not found or plus wasn't true.")
        if not errormsg:
            logging.info("no error message. Returning empty.")
            return Empty()
        else:
            logging.info("There is an error message. Raising exception.")
        raise Error(errormsg)

    @log
    def if_not_found(self, plus=True, errormsg=''):
        if not self.found and plus:
            logging.info("%s was not found and plus was true.", self.picturename)
            return self
        else:
            logging.info("%s was found or plus was not true.", self.picturename)
        if not errormsg:
            logging.info("no error message. Returning empty.")
            return Empty()
        else:
            logging.info("There is an error message. Raising exception.")
        raise Error(errormsg)

    @log
    def fail(self, errormsg):
        raise Error(errormsg)

    @log
    def keywrite(self, *args):
        gi.typewrite(list(args),interval=0.2)

    @log
    def write(self, sentence):
        gi.typewrite(sentence, interval=0.2)

    @log
    def screenshot(self, offset=None, width=None, height=None):
        start = gi.locateOnScreen(self.url)
        x, y = start[0] + offset[0], start[1] + offset[1]
        logging.info("Final x and y is %d, %d", x, y)
        return gi.screenshot("../screen.png", region=(x, y, width, height))

    @log
    def read(self, offset=None, width=None, height=None):
        img = self.screenshot(offset=(90,5), width=100, height=12)
        recg = str(ocr.make_recognition(img).rstrip())
        return recg


Items = {}



from distutils.core import setup

setup(
    name='spookyaction',
    version='0.0.4',
    author='Adam Short',
    author_email='adam010101@icloud.com',
    packages=['spookyaction'],
    url='http://pypi.python.org/pypi/TowelStuff/',
    license='LICENSE.txt',
    description='A framework for pyautogui automation.',
    long_description=open('README.txt').read(),
)
